@@ -394,8 +394,8 @@ def gather_vswhere_versions(conf, versions):
 
 	prg_path = os.environ.get('ProgramFiles(x86)', os.environ.get('ProgramFiles', 'C:\\Program Files (x86)'))
 
-	vswhere = os.path.join(prg_path, 'Microsoft Visual Studio', 'Installer', 'vswhere.exe')
-	args = [vswhere, '-products', '*', '-legacy', '-format', 'json']
+	vswhere = conf.env.VSWHERE or [os.path.join(prg_path, 'Microsoft Visual Studio', 'Installer', 'vswhere.exe')]
+	args = vswhere + ['-products', '*', '-legacy', '-format', 'json']
 
 	if conf.options.msvc_prerelease:
 		args.append('-prerelease')
