@@ -8,46 +8,6 @@ Filesystem-based cache system to share and re-use build artifacts
 Cache access operations (copy to and from) are delegated to
 independent pre-forked worker subprocesses.
 
-The following environment variables may be set:
-* WAFCACHE: several possibilities:
-  - File cache:
-    absolute path of the waf cache (~/.cache/wafcache_user,
-    where `user` represents the currently logged-in user)
-  - URL to a cache server, for example:
-    export WAFCACHE=http://localhost:8080/files/
-    in that case, GET/POST requests are made to urls of the form
-    http://localhost:8080/files/000000000/0 (cache management is delegated to the server)
-  - GCS, S3 or MINIO bucket
-    gs://my-bucket/    (uses gsutil command line tool or WAFCACHE_CMD)
-    s3://my-bucket/    (uses aws command line tool or WAFCACHE_CMD)
-    minio://my-bucket/ (uses mc command line tool or WAFCACHE_CMD)
-* WAFCACHE_CMD: bucket upload/download command, for example:
-    WAFCACHE_CMD="gsutil cp %{SRC} %{TGT}"
-  Note that the WAFCACHE bucket value is used for the source or destination
-  depending on the operation (upload or download). For example, with:
-    WAFCACHE="gs://mybucket/"
-  the following commands may be run:
-    gsutil cp build/myprogram  gs://mybucket/aa/aaaaa/1
-    gsutil cp gs://mybucket/bb/bbbbb/2 build/somefile
-* WAFCACHE_NO_PUSH: if set, disables pushing to the cache
-* WAFCACHE_VERBOSITY: if set, displays more detailed cache operations
-* WAFCACHE_STATS: if set, displays cache usage statistics on exit
-
-File cache specific options:
-  Files are copied using hard links by default; if the cache is located
-  onto another partition, the system switches to file copies instead.
-* WAFCACHE_TRIM_MAX_FOLDER: maximum amount of tasks to cache (1M)
-* WAFCACHE_EVICT_MAX_BYTES: maximum amount of cache size in bytes (10GB)
-* WAFCACHE_EVICT_INTERVAL_MINUTES: minimum time interval to try
-                                   and trim the cache (3 minutes)
-
-Upload specific options:
-* WAFCACHE_ASYNC_WORKERS: define a number of workers to upload results asynchronously
-                          this may improve build performance with many/long file uploads
-                          the default is unset (synchronous uploads)
-* WAFCACHE_ASYNC_NOWAIT: do not wait for uploads to complete (default: False)
-                         this requires asynchonous uploads to have an effect
-
 Usage::
 
 	def build(bld):
@@ -57,6 +17,71 @@ Usage::
 To troubleshoot::
 
 	waf clean build --zone=wafcache
+
+General parameters
+^^^^^^^^^^^^^^^^^^
+
+Use the following environment variables
+
+* WAFCACHE, which can be
+
+  - An absolute path to the waf cache folder, by default::
+
+      ~/.cache/wafcache_user # `user` represents the currently logged-in user
+
+  - A URL to a cache server, for example::
+
+      export WAFCACHE=http://localhost:8080/files/
+      #in that case, `GET/POST` requests are made to urls of the form
+      #`http://localhost:8080/files/000000000/0` (cache management is delegated to the server)
+
+  - A GCS, S3 or MINIO bucket::
+
+      gs://my-bucket/    # (uses gsutil command line tool or WAFCACHE_CMD)
+      s3://my-bucket/    # (uses aws command line tool or WAFCACHE_CMD)
+      minio://my-bucket/ # (uses mc command line tool or WAFCACHE_CMD)
+
+* WAFCACHE_CMD: bucket upload/download command, for example::
+
+    WAFCACHE_CMD="gsutil cp %{SRC} %{TGT}"
+
+* WAFCACHE_NO_PUSH: if set, disables pushing to the cache
+* WAFCACHE_VERBOSITY: if set, displays more detailed cache operations
+* WAFCACHE_STATS: if set, displays cache usage statistics on exit
+
+Remote buckets
+^^^^^^^^^^^^^^
+
+The WAFCACHE bucket value is used for the source or destination
+depending on the operation (upload or download). For example, with::
+
+    WAFCACHE="gs://mybucket/"
+
+the following commands may be run::
+
+    gsutil cp build/myprogram  gs://mybucket/aa/aaaaa/1
+    gsutil cp gs://mybucket/bb/bbbbb/2 build/somefile
+
+File cache
+^^^^^^^^^^
+
+Files are copied using hard links by default; if the cache is located
+onto another partition, the system switches to file copies instead.
+
+Additional environments can be set:
+
+* WAFCACHE_TRIM_MAX_FOLDER: maximum amount of tasks to cache (1M)
+* WAFCACHE_EVICT_MAX_BYTES: maximum amount of cache size in bytes (10GB)
+* WAFCACHE_EVICT_INTERVAL_MINUTES: minimum time interval to try and trim the cache (3 minutes)
+
+Asynchronous transfers
+^^^^^^^^^^^^^^^^^^^^^^
+
+* WAFCACHE_ASYNC_WORKERS: define a number of workers to upload results asynchronously
+                          this may improve build performance with many/long file uploads
+                          the default is unset (synchronous uploads)
+* WAFCACHE_ASYNC_NOWAIT: do not wait for uploads to complete (default: False)
+                         this requires asynchonous uploads to have an effect
 """
 
 import atexit, base64, errno, getpass, os, re, shutil, sys, time, threading, traceback, shlex
@@ -212,7 +237,7 @@ def hash_env_vars(self, env, vars_lst):
 
 	v = str([env[a] for a in vars_lst])
 	v = v.replace(self.srcnode.abspath().__repr__()[:-1], '')
-	m = Utils.md5()
+	m = Utils.md5(usedforsecurity=False)
 	m.update(v.encode())
 	ret = m.digest()
 
@@ -229,7 +254,7 @@ def uid(self):
 	try:
 		return self.uid_
 	except AttributeError:
-		m = Utils.md5()
+		m = Utils.md5(usedforsecurity=False)
 		src = self.generator.bld.srcnode
 		up = m.update
 		up(self.__class__.__name__.encode())
@@ -468,6 +493,8 @@ def lru_evict():
 
 				kernel32 = ctypes.windll('kernel32', use_last_error=True)
 				DWORD = ctypes.wintypes.DWORD
+				BOOL = ctypes.wintypes.BOOL
+				POINTER = ctypes.POINTER
 				HANDLE = ctypes.wintypes.HANDLE
 				class DUMMYSTRUCTNAME(ctypes.Structure):
 					_fields = [('Offset', ctypes.wintypes.DWORD), ('OffsetHigh', DWORD)]
@@ -489,7 +516,7 @@ def lru_evict():
 						lru_trim()
 						os.utime(lockfile, None)
 					finally:
-						win32file.UnlockFileEx(handle, 0, 1, 0, ctypes.pointer(OVERLAPPED()))
+						UnlockFileEx(handle, 0, 1, 0, ctypes.pointer(OVERLAPPED()))
 				else:
 					last_error = kernel32.GetLastError()
 					if last_error == 33:
